Package: luz
Title: Higher Level 'API' for 'torch'
Version: 0.5.2
Authors@R: c(
    person("Tomasz", "Kalinowski", role = c("ctb", "cre"), email = "tomasz@posit.co"),
    person("Daniel", "Falbel", email = "dfalbel@gmail.com", role = c("aut", "cph")),
    person("Christophe", "Regouby", email = "christophe.regouby@free.fr", role = c("ctb")),
    person(family = "RStudio", role = c("cph"))
    )
Description: A high level interface for 'torch' providing utilities to reduce the
    the amount of code needed for common tasks, abstract away torch details and 
    make the same code work on both the 'CPU' and 'GPU'. It's flexible enough to
    support expressing a large range of models. It's heavily inspired by 'fastai' by 
    Howard et al. (2020) <doi:10.48550/arXiv.2002.04688>, 'Keras' by Chollet et al. (2015) and 
    'PyTorch Lightning' by Falcon et al. (2019) <doi:10.5281/zenodo.3828935>.
License: MIT + file LICENSE
URL: https://mlverse.github.io/luz/, https://github.com/mlverse/luz
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: torch (>= 0.11.9000), magrittr, zeallot, rlang (>= 1.0.0),
        coro, glue, progress, R6, generics, purrr, fs, prettyunits, cli
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), covr, Metrics, withr,
        vdiffr, ggplot2 (>= 3.0.0), dplyr, torchvision, tfevents (>=
        0.0.2), tidyr
VignetteBuilder: knitr
Config/testthat/edition: 3
Collate: 'accelerator.R' 'as_dataloader.R' 'utils.R' 'callbacks.R'
        'callbacks-amp.R' 'callbacks-interrupt.R' 'callbacks-mixup.R'
        'callbacks-monitor-metrics.R' 'callbacks-profile.R'
        'callbacks-resume.R' 'callbacks-tfevents.R' 'context.R'
        'losses.R' 'lr-finder.R' 'metrics.R' 'metrics-auc.R'
        'module-plot.R' 'module-print.R' 'module.R' 'reexports.R'
        'serialization.R'
BugReports: https://github.com/mlverse/luz/issues
NeedsCompilation: no
Packaged: 2026-04-27 22:20:18 UTC; dfalbel
Author: Tomasz Kalinowski [ctb, cre],
  Daniel Falbel [aut, cph],
  Christophe Regouby [ctb],
  RStudio [cph]
Maintainer: Tomasz Kalinowski <tomasz@posit.co>
Repository: CRAN
Date/Publication: 2026-04-28 08:10:08 UTC
Built: R 4.6.0; ; 2026-04-28 11:13:56 UTC; unix
